/**
 * Copyright (c) 2017-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @noflow
 * @format
 */
'use strict';

/* eslint nuclide-internal/no-commonjs: 0 */

const DISALLOWED_MODULES = new Map([
  ['invariant', '"assert"'],
  // TODO(hansonw): Remove this if this bugfix PR gets merged:
  // https://github.com/substack/node-shell-quote/pull/29
  ['shell-quote', 'shellQuote and shellParse from nuclide-commons/string'],
]);

module.exports = function(context) {
  return {
    ImportDeclaration(node) {
      const name = node.source.value;
      const descriptionOfAlternative = DISALLOWED_MODULES.get(name);
      if (descriptionOfAlternative != null) {
        context.report({
          node,
          message: `Use ${descriptionOfAlternative} instead of "${name}"`,
        });
      }
    },
  };
};
