"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parsePsTableOutput = parsePsTableOutput;

/**
 * Copyright (c) 2017-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * 
 * @format
 */
function parsePsTableOutput(output, desiredFields) {
  const lines = output.split(/\n/);
  const header = lines[0];
  const cols = header.split(/\s+/);
  const colMapping = {};

  for (let i = 0; i < cols.length; i++) {
    const columnName = cols[i].toLowerCase();

    if (desiredFields.includes(columnName)) {
      colMapping[i] = columnName;
    }
  }

  const formattedData = [];
  const data = lines.slice(1);
  const ignoreSColumn = cols.find(col => col.trim() === 'S') == null;
  data.filter(row => row.trim() !== '').forEach(row => {
    const rowData = row.split(/\s+/);
    const rowObj = {};

    for (let i = 0; i < rowData.length; i++) {
      const effectiveColumn = i;

      if (ignoreSColumn) {
        // Android's ps output has an extra column "S" (versions prior to API 26)
        // in the data that doesn't appear in the header. Skip that column's value.
        if (rowData[i] === 'S' && i < rowData.length - 1) {
          i++;
        }
      }

      if (colMapping[effectiveColumn] !== undefined) {
        rowObj[colMapping[effectiveColumn]] = rowData[i];
      }
    }

    formattedData.push(rowObj);
  });
  return formattedData;
}