"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _atom = require("atom");

function _UniversalDisposable() {
  const data = _interopRequireDefault(require("../../../modules/nuclide-commons/UniversalDisposable"));

  _UniversalDisposable = function () {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the license found in the LICENSE file in
 * the root directory of this source tree.
 *
 * 
 * @format
 */
class QuickOpenProviderRegistry {
  constructor() {
    this._emitter = new _atom.Emitter();
    this._subscriptions = new (_UniversalDisposable().default)();
    this._directoryProviders = new Map();
    this._globalProviders = new Map();
  }

  getProviders() {
    return [...this._globalProviders.values(), ...this._directoryProviders.values()];
  }

  getGlobalProviders() {
    return [...this._globalProviders.values()];
  }

  getDirectoryProviders() {
    return [...this._directoryProviders.values()];
  }

  getProviderByName(serviceName) {
    return this._globalProviders.get(serviceName) || this._directoryProviders.get(serviceName);
  }

  getGlobalProviderByName(serviceName) {
    return this._globalProviders.get(serviceName);
  }

  getDirectoryProviderByName(serviceName) {
    return this._directoryProviders.get(serviceName);
  }

  isProviderGlobal(serviceName) {
    return this._globalProviders.has(serviceName);
  }

  observeProviders(callback) {
    for (const provider of this.getProviders()) {
      callback(provider);
    }

    return this.onDidAddProvider(callback);
  }

  onDidAddProvider(callback) {
    return this._emitter.on('did-add-provider', callback);
  }

  onDidRemoveProvider(callback) {
    return this._emitter.on('did-remove-provider', callback);
  }

  addProvider(service) {
    if (service.providerType === 'GLOBAL') {
      this._globalProviders.set(service.name, service);
    } else {
      this._directoryProviders.set(service.name, service);
    }

    const disposable = new (_UniversalDisposable().default)(() => {
      if (service.providerType === 'GLOBAL') {
        this._globalProviders.delete(service.name);
      } else {
        this._directoryProviders.delete(service.name);
      }

      this._emitter.emit('did-remove-provider', service);
    });

    this._subscriptions.add(disposable);

    this._emitter.emit('did-add-provider', service);

    return disposable;
  }

  dispose() {
    this._emitter.dispose();

    this._subscriptions.dispose();
  }

}

exports.default = QuickOpenProviderRegistry;